package Anarres::Mud::Driver::Efun::MudOS;

use strict;
use vars qw(@ISA);
use Anarres::Mud::Driver::Program::Type qw(:types);
use Anarres::Mud::Driver::Program::Node qw(:flags);
use Anarres::Mud::Driver::Program::Efun qw(register);

# Efuns need to be normal functions in a program symbol table but
# will not issue a warning if multiply defined as efuns.
# In fact, they probably won't inherit at all. They only resolve at
# link time.

@ISA = qw(Anarres::Mud::Driver::Program::Efun);

{
	# As traditional, [ flags, return type, argtype .... ]
	my %efuns = (
		debug_message		=> [ 0,		T_STRING, T_STRING, ],
		previous_object		=> [ 0,		T_OBJECT, T_INTEGER, ],
		file_name			=> [ PURE,	T_STRING, T_OBJECT, ],
		find_object			=> [ PURE,	T_OBJECT, T_STRING, ],
		to_string			=> [ PURE,	T_STRING, T_INTEGER, ],

		call_stack			=> [ 0,		T_ARRAY,  T_INTEGER, ],
			);

	# We call this as an exported function since ISA isn't yet set up.
	foreach (keys %efuns) {
		register(__PACKAGE__ . "::" . $_, @{ $efuns{$_} });
	}
}

{
	package Anarres::Mud::Driver::Efun::MudOS::debug_message;
	sub gencall { "print STDERR $_[1], '\\n'" }
}

{
	package Anarres::Mud::Driver::Efun::MudOS::previous_object;
	sub invoke { undef }
}

{
	package Anarres::Mud::Driver::Efun::MudOS::file_name;
	sub gencall {
		"Anarres::Mud::Driver::Program::package_to_path(ref($_[1]))"
	}
}

{
	package Anarres::Mud::Driver::Efun::MudOS::find_object;
	# sub gencall { "undef" }
	sub invoke { undef }
}

{
	package Anarres::Mud::Driver::Efun::MudOS::to_string;
	# XXX This only works for CONSTANT integers, of course.
	# sub gencall { '"' . $_[1] . '"' }
	# This works for any integer which is about to be evaluated as
	# a string by Perl. 6 . 7 == "67";
	sub gencall { ('' . $_[1]) }
}

1;
